@@ -1,24 +1,29 @@
-/***************************************************************************
-                          alphabet.c 
+/*
+   alphabet.c:
 
-                             -------------------
-    begin                : Jan 6 2003
-    copyright            : (C) 2003 by Jesse Andrews
-    email                : jdandr2@tux4kids.net
-Revised extensively: 2008
-Sreyas Kurumanghat <k.sreyas@gmail.com>
-and
-David Bruce <davidstuartbruce@gmail.com>
- ***************************************************************************/
+   Code for word lists, keyboard fingering, and related issues.
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+   Copyright 2003, 2008, 2009, 2010.
+   Authors: Jesse Andrews, Sreyas Kurumanghat, David Bruce.
+   
+   Project email: <tux4kids-tuxtype-dev@lists.alioth.debian.org>
+   Project website: http://tux4kids.alioth.debian.org
+
+   alphabet.c is part of Tux Typing, a.k.a "tuxtype".
+
+Tux Typing is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+Tux Typing is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 
 
@@ -558,12 +563,15 @@ int GenerateWordList(const char* wordFn)
       continue;
     }
 
-    if (length > MAX_WORD_SIZE)
-    {
-      fprintf(stderr, "Word '%s' not added - exceeds %d characters\n",
-              temp_word, MAX_WORD_SIZE);
-      continue;
-    }
+    //NOTE now the cascade and laser games have their own checks for how long of words
+    //they can actually handle, so we let long words in now - DSB
+
+//     if (length > MAX_WORD_SIZE)
+//     {
+//       fprintf(stderr, "Word '%s' not added - exceeds %d characters\n",
+//               temp_word, MAX_WORD_SIZE);
+//       continue;
+//     }
 
     if (num_words >= MAX_NUM_WORDS)
     {
@@ -1077,83 +1085,83 @@ int map_keys(wchar_t wide_char, kbd_char* keyboard_entry)
 		keyboard_entry->shift=1;
 		keyboard_entry->finger=9;
 		break;
-	case 'z':strcpy(keyboard_entry->keyname,"D01");
+	case 'z':strcpy(keyboard_entry->keyname,"D02");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=0;
 		break;
-	case 'Z':strcpy(keyboard_entry->keyname,"D01");
+	case 'Z':strcpy(keyboard_entry->keyname,"D02");
 		keyboard_entry->shift=2;
 		keyboard_entry->finger=0;
 		break;
-	case 'x':strcpy(keyboard_entry->keyname,"D02");
+	case 'x':strcpy(keyboard_entry->keyname,"D03");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=1;
 		break;
-	case 'X':strcpy(keyboard_entry->keyname,"D02");
+	case 'X':strcpy(keyboard_entry->keyname,"D03");
 		keyboard_entry->shift=2;
 		keyboard_entry->finger=1;
 		break;
-	case 'c':strcpy(keyboard_entry->keyname,"D03");
+	case 'c':strcpy(keyboard_entry->keyname,"D04");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=2;
 		break;
-	case 'C':strcpy(keyboard_entry->keyname,"D03");
+	case 'C':strcpy(keyboard_entry->keyname,"D04");
 		keyboard_entry->shift=2;
 		keyboard_entry->finger=2;
 		break;
-	case 'v':strcpy(keyboard_entry->keyname,"D04");
+	case 'v':strcpy(keyboard_entry->keyname,"D05");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=3;
 		break;
-	case 'V':strcpy(keyboard_entry->keyname,"D04");
+	case 'V':strcpy(keyboard_entry->keyname,"D05");
 		keyboard_entry->shift=2;
 		keyboard_entry->finger=3;
 		break;
-	case 'b':strcpy(keyboard_entry->keyname,"D05");
+	case 'b':strcpy(keyboard_entry->keyname,"D06");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=3;
 		break;
-	case 'B':strcpy(keyboard_entry->keyname,"D05");
+	case 'B':strcpy(keyboard_entry->keyname,"D06");
 		keyboard_entry->shift=2;
 		keyboard_entry->finger=3;
 		break;
-	case 'n':strcpy(keyboard_entry->keyname,"D06");
+	case 'n':strcpy(keyboard_entry->keyname,"D07");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=6;
 		break;
-	case 'N':strcpy(keyboard_entry->keyname,"D06");
+	case 'N':strcpy(keyboard_entry->keyname,"D07");
 		keyboard_entry->shift=1;
 		keyboard_entry->finger=6;
 		break;
-	case 'm':strcpy(keyboard_entry->keyname,"D07");
+	case 'm':strcpy(keyboard_entry->keyname,"D08");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=6;
 		break;
-	case 'M':strcpy(keyboard_entry->keyname,"D07");
+	case 'M':strcpy(keyboard_entry->keyname,"D08");
 		keyboard_entry->shift=1;
 		keyboard_entry->finger=6;
 		break;
-	case ',':strcpy(keyboard_entry->keyname,"D08");
+	case ',':strcpy(keyboard_entry->keyname,"D09");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=7;
 		break;
-	case '<':strcpy(keyboard_entry->keyname,"D08");
+	case '<':strcpy(keyboard_entry->keyname,"D09");
 		keyboard_entry->shift=1;
 		keyboard_entry->finger=7;
 		break;
-	case '.':strcpy(keyboard_entry->keyname,"D09");
+	case '.':strcpy(keyboard_entry->keyname,"D010");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=8;
 		break;
-	case '>':strcpy(keyboard_entry->keyname,"D09");
+	case '>':strcpy(keyboard_entry->keyname,"D010");
 		keyboard_entry->shift=1;
 		keyboard_entry->finger=8;
 		break;
-	case '/':strcpy(keyboard_entry->keyname,"D10");
+	case '/':strcpy(keyboard_entry->keyname,"D11");
 		keyboard_entry->shift=0;
 		keyboard_entry->finger=9;
 		break;
-	case '?':strcpy(keyboard_entry->keyname,"D10");
+	case '?':strcpy(keyboard_entry->keyname,"D11");
 		keyboard_entry->shift=1;
 		keyboard_entry->finger=9;
 		break;
@@ -1219,8 +1227,8 @@ void GenerateKeyboard(SDL_Surface* keyboard)
       case 'A': new.y += 6;   new.x += 12; break;
       case 'B': new.y += 39;  new.x += 23; break;
       case 'C': new.y += 69;  new.x += 33; break;
-//      case 'D': new.y += 96;  new.x += 23; break;
-      case 'D': new.y += 99;  new.x += 54; break;
+      case 'D': new.y += 100;  new.x += 21; break;
+//      case 'D': new.y += 99;  new.x += 54; break;
       case 'E': new.y += 129; break;
       default: render = 0; break;
     }
@@ -1228,7 +1236,7 @@ void GenerateKeyboard(SDL_Surface* keyboard)
     if(!render)
       continue;
 
-    new.x += (31 * col);
+    new.x += (int)(30.5 * col);
 
     if(keyboard_list[i].shift > 0)
     {
