@@ -1,21 +1,32 @@
-/***************************************************************************
- -  file: laser.c
- -  description: a modification of TuxMath for typing :)
-                            -------------------
-    begin                : 
-    copyright            : Bill Kendrick (C) 2002
-                           Jesse Andrews (C) 2003
-    email                : tuxtype-dev@tux4kids.net
-***************************************************************************/
+/*
+   laser.h:
+
+   "Comet Zap" typing game, adapted from "Tux, of Math
+   Command"  (http://tux4kids.alioth.debian.org)
+   
+   Copyright 2000, 2003, 2008, 2010.
+   Authors: Bill Kendrick, Jesse Andrews, David Bruce.
+   
+   Project email: <tux4kids-tuxtype-dev@lists.alioth.debian.org>
+   Project website: http://tux4kids.alioth.debian.org
+
+   laser.h is part of Tux Typing, a.k.a "tuxtype".
+
+Tux Typing is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+Tux Typing is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 
-/***************************************************************************
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-***************************************************************************/
 
 #include "globals.h"
 #include "funcs.h"
@@ -194,6 +205,8 @@ int PlayLaserGame(int diff_level)
                                   key_unicode -= 32;  //convert lowercase to uppercase
                                 if (key_unicode >= 224 && key_unicode <= 255)
                                   key_unicode -= 32; //same for non-US chars
+                                if ((key_unicode >= 256) && (key_unicode <= 382))  // Fix for other letters, such as the hungarian letter O with double acute
+                                    key_unicode -= 1;
 
 				LOG ("After checking for lower case:\n");
 				DEBUGCODE
@@ -318,7 +331,7 @@ int PlayLaserGame(int diff_level)
 			while (tux_img == old_tux_img)
 				tux_img = IMG_TUX_CONSOLE1 + (rand() % 3);
 
-			PlaySound(sounds[SND_CLICK]);
+			PlaySound(sounds[SND_TOCK]);
 		}
       
       
@@ -368,11 +381,11 @@ int PlayLaserGame(int diff_level)
 					if (cities[comets[i].city].shields) {
 						cities[comets[i].city].shields = 0;
 						PlaySound(sounds[SND_SHIELDSDOWN]);
-						laser_add_score(-500 * (diff_level+1));
+						laser_add_score(-50 * (diff_level+1));
 					} else {
 						cities[comets[i].city].expl = CITY_EXPL_START;
 						PlaySound(sounds[SND_EXPLOSION]);
-						laser_add_score(-1000 * (diff_level+1));
+						laser_add_score(-100 * (diff_level+1));
 					}
 
 					tux_anim = IMG_TUX_FIST1;
@@ -867,9 +880,11 @@ static void laser_add_comet(int diff_level)
           int i = 0;
           comet_type* prev_comet = NULL;
 
-          if(!word || wcslen(word) == 0)
+          if(!word
+             || (wcslen(word) == 0)
+             || (wcslen(word) > NUM_CITIES - 1))
           {
-            fprintf(stderr, "Error - GetWord() returned NULL or zero-length word\n");
+            fprintf(stderr, "Error - GetWord() returned NULL, zero-length, or too-long word\n");
             return; 
           }
 
