@@ -1,32 +1,43 @@
-/***************************************************************************
-playgame.c 
--  description: Cascade game
--------------------
-begin                : Fri May 5 2000
-copyright            : (C) 2000 by Sam Hart
-                     : (C) 2003 by Jesse Andrews
-email                : tuxtype-dev@tux4kids.net
-***************************************************************************/
+/*
+   playgame.c:
+
+   Code for "Fish Cascade" typing activity.
+
+   Copyright 2000, 2003, 2007, 2008, 2009, 2010, 2011.
+   Authors: Sam Hart, Jesse Andrews, David Bruce, George Stephanos.
+   
+   Project email: <tux4kids-tuxtype-dev@lists.alioth.debian.org>
+   Project website: http://tux4kids.alioth.debian.org
+
+   playgame.c is part of Tux Typing, a.k.a "tuxtype".
+
+Tux Typing is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+Tux Typing is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+
 
-/***************************************************************************
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-***************************************************************************/
 
 #include "globals.h"
 #include "funcs.h"
 #include "playgame.h"
 #include "snow.h"
 #include "SDL_extras.h"
-
+#include "input_methods.h"
 
 /* Should these be constants? */
-static int tux_max_width;                // the max width of the images of tux
-static int number_max_w;                 // the max width of a number image
+static int tux_max_width = 0;                // the max width of the images of tux
+static int number_max_w = 0;                 // the max width of a number image
 
 
 //static SDL_Surface* background = NULL;
@@ -42,8 +53,8 @@ static Mix_Chunk* sound[NUM_WAVES];
 static sprite* fish_sprite = NULL;
 static sprite* splat_sprite = NULL;
 
-
-
+/* For input_methods.c processing of Asian text input: */
+static IM_DATA im_data;
 
 
 /* Local function prototypes: */
@@ -61,10 +72,12 @@ static void EraseNumbers(int num, int x, int y, int places);
 
 static float float_restrict(float a, float x, float b);
 static void FreeGame(void);
+//static void HandleKey(SDL_keysym* key_sym);
+static void HandleKey(SDL_keysym* key_sym);
 static int int_restrict(int a, int x, int b);
 static void LoadFishies(void);
 static void LoadOthers(void);
-static void LoadTuxAnims(void);
+static int LoadTuxAnims(void);
 static void MoveFishies(int* fishies, int* splats, int* lifes, int* frame);
 static void MoveTux(int frame, int fishies);
 static void next_tux_frame(void);
@@ -93,8 +106,8 @@ int PlayCascade(int diflevel)
   int won_level = 0;
   int quitting = 0;
   int curlevel = 0;
-  int i;
-  int curlives;
+  int i = 0;
+  int curlives = 0;
   int oldlives = 0;
   int oldfish_left = 0;
   int fish_left = 0;
@@ -121,14 +134,24 @@ int PlayCascade(int diflevel)
     fprintf(stderr, "->Entering PlayCascade(): level=%i\n", diflevel);
   }
 
-  SDL_ShowCursor(0);
+//  SDL_ShowCursor(0); //don't really need this and it causes a bug on windows
 
 //	SNOW_init();
 
-  LoadTuxAnims(); 
+  if(!LoadTuxAnims())
+  {
+    fprintf(stderr, "PlayCascade() - LoadTuxAnims() failed - returning to menu!\n\n\n");
+    FreeGame();
+    return 0;
+  }
+  
   LoadFishies();
   LoadOthers();
 
+  /* Initialize input_methods system: */
+//  im_init(&im_data, get_current_language());
+  im_init(&im_data, 0); //will need function to put in correct language code:
+
   /* Make sure everything in the word list is "typable" according to the current */
   /* theme's keyboard.lst:                                                       */
   if (!CheckNeededGlyphs())
@@ -192,8 +215,9 @@ int PlayCascade(int diflevel)
       if (diflevel == INF_PRACT)
         sprintf(filename, "pract.png");
       else
-        sprintf(filename, "kcas%i_%i.jpg", diflevel+1, curlevel+1);
-
+      {	
+        sprintf(filename, "kcas%d.jpg", rand() % 12);
+      }
       /* ---  Special Hidden Code  --- */
 
       if (settings.hidden && curlevel == 3)
@@ -212,7 +236,11 @@ int PlayCascade(int diflevel)
 
       if (settings.sys_sound)
       {
-        sprintf(filename, "kmus%i.wav", curlevel + 1);
+        //TODO make use of more music files
+        if(rand() % 2) 
+          sprintf(filename, "amidst_the_raindrops.ogg");
+        else
+          sprintf(filename, "chiptune2.ogg");
         MusicLoad( filename, -1 );
       }
 
@@ -245,87 +273,91 @@ int PlayCascade(int diflevel)
         {
           if (event.type == SDL_KEYDOWN)
           {
-            if (event.key.keysym.sym == SDLK_F11) 
-              SDL_SaveBMP(screen, "screenshot.bmp");
-
-            if (event.key.keysym.sym == SDLK_F6)
+            switch (event.key.keysym.sym)
             {
-              settings.o_lives = settings.o_lives - 10;
-              curlives = curlives - 10;
-            }
+              case SDLK_F11:
+                SDL_SaveBMP(screen, "screenshot.bmp");
+                break;
 
-            if (event.key.keysym.sym == SDLK_F7)
-            {
-              settings.o_lives = settings.o_lives + 10;
-              curlives = curlives + 10;
-            }
+              case SDLK_F6:
+                settings.o_lives = settings.o_lives - 10;
+                curlives = curlives - 10;
+                break;
 
-            /* FIXME could be improved - currently clears out all words    */
-            /* in play at time of switch, so could be used to avoid losing */
-            if (event.key.keysym.sym == SDLK_F10)
-            {
-              /* first wipe out old blits because screen size is changing */
-              /* and otherwise we would segfault:                         */
-              ResetBlitQueue();
-              //numupdates = 0;
+              case SDLK_F7:
+                settings.o_lives = settings.o_lives + 10;
+                curlives = curlives + 10;
+                break;
 
-              SwitchScreenMode();
-
-              DrawBackground();
-              ResetObjects();
-
-            }
-
-            if (event.key.keysym.sym == SDLK_F12)
-            {
-//              SNOW_toggle();
-            }
-
-            if (event.key.keysym.sym == SDLK_ESCAPE)
-            {
-              /* Pause() returns 1 if quitting, */
-              /* 0 if returning to game:        */
-              if (Pause() == 1)
-              {
-                playing_level = 0;
-                still_playing = 0;
-                quitting = 1;
-              }
-              else  /* Returning to game */
+              case SDLK_F10:
+                /* NOTE this could be used to "cheat" by wiping out all the */
+                /* current words if the player is about to lose.            */
+                /* first wipe out old blits because screen size is changing */
+                /* and otherwise we would segfault:                         */
+                ResetBlitQueue();
+                //numupdates = 0;
+                SwitchScreenMode();
                 DrawBackground();
+                ResetObjects();
+                break;
+
+              case SDLK_F12:
+                // SNOW_toggle();
+                break;
+
+              case SDLK_ESCAPE:
+                /* Pause() returns 1 if quitting, */
+                /* 0 if returning to game:        */
+                if (Pause() == 1)
+                {
+                  playing_level = 0;
+                  still_playing = 0;
+                  quitting = 1;
+                }
+                else  /* Returning to game */
+                  DrawBackground();
+                break;
+
+              /*  Don't count modifier keys as keystrokes in game: */ 
+	      case SDLK_RSHIFT:
+	      case SDLK_LSHIFT:
+              case SDLK_RCTRL:
+	      case SDLK_LCTRL:
+	      case SDLK_RALT: 
+              case SDLK_LALT:
+	      case SDLK_RMETA:
+	      case SDLK_LMETA:
+              case SDLK_LSUPER:
+              case SDLK_RSUPER:
+                break;
+
+              default:
+              /*----------------------------------------------------*/
+              /* Some other key - player is actually typing!!!!!!!! */
+              /*----------------------------------------------------*/
+
+                /* See what Unicode value was typed: */
+                key_unicode = event.key.keysym.unicode;
+
+                DEBUGCODE
+                {fprintf(stderr, "\nkey_unicode = %d\twchar_t = %lc\t\n", key_unicode, key_unicode);}
+
+                /* For now, the cascade game is case-insensitive for input, */
+                /* with only uppercase for answers:                         */
+                if (key_unicode >= 97 && key_unicode <= 122)
+                  key_unicode -= 32;  //convert lowercase to uppercase
+                if (key_unicode >= 224 && key_unicode <= 255)
+                  key_unicode -= 32; //same for non-US Western European chars
+                if ((key_unicode >= 256) && (key_unicode <= 382))  // Fix for other letters, such as the hungarian letter O with double acute
+                    key_unicode -= 1;
+
+                LOG ("After checking for lower case:\n");
+                DEBUGCODE
+                {fprintf(stderr, "key_unicode = %d\twchar_t = %lc\\n\n", key_unicode, key_unicode);}
+
+                /* Now update with case-folded value: */
+                UpdateTux(key_unicode, fishies, frame);
             }
-
-            /*----------------------------------------------------*/
-            /* Some other key - player is actually typing!!!!!!!! */
-            /*----------------------------------------------------*/
-
-            /* See what Unicode value was typed: */
-            key_unicode = event.key.keysym.unicode;
-
-            DEBUGCODE
-            {
-              fprintf(stderr,
-                      "\nkey_unicode = %d\twchar_t = %lc\t\n",
-                      key_unicode, key_unicode);
-            }
-
-            /* For now, the cascade game is case-insensitive for input, */
-            /* with only uppercase for answers:                         */
-            if (key_unicode >= 97 && key_unicode <= 122)
-              key_unicode -= 32;  //convert lowercase to uppercase
-            if (key_unicode >= 224 && key_unicode <= 255)
-              key_unicode -= 32; //same for non-US chars
-
-            LOG ("After checking for lower case:\n");
-            DEBUGCODE
-            {
-              fprintf(stderr,
-                      "key_unicode = %d\twchar_t = %lc\\n\n",
-                      key_unicode, key_unicode);
-            }
-
-            /* Now update with case-folded value: */
-            UpdateTux(key_unicode, fishies, frame);
           }
         }
       }   /* ------ End of 'while' loop for handling user input ------- */
@@ -398,7 +430,7 @@ int PlayCascade(int diflevel)
       }
       else
       {
-        fprintf(stderr, "Did not achieve desired frame rate!\n");
+        LOG("Did not achieve desired frame rate!\n");
       }
     }  /* End per-frame game loop - level completed */
 
@@ -631,7 +663,7 @@ static void LoadOthers(void)
 	number_max_w = 0;
 	for (i = 0; i < NUM_NUMS; i++) {
 		sprintf(filename, "num_%i.png", i);
-		number[i] = LoadImage( filename, IMG_COLORKEY );
+		number[i] = LoadImage( filename, IMG_ALPHA );
 		if (number[i]->w > number_max_w)
 			number_max_w = number[i]->w;
 	}
@@ -702,8 +734,8 @@ static void LoadFishies(void)
 
 	LOG( "=LoadFishies()\n" );
 
-	fish_sprite = LoadSprite( "fishy", IMG_COLORKEY );
-	splat_sprite = LoadSprite( "splat", IMG_COLORKEY );
+	fish_sprite = LoadSprite( "fishy", IMG_ALPHA );
+	splat_sprite = LoadSprite( "splat", IMG_ALPHA );
 
 	for (i = 0; i < MAX_FISHIES_HARD; i++) {
 		fish_object[i].alive = 1;
@@ -717,22 +749,31 @@ static void LoadFishies(void)
 /******************************
 LoadTuxAnims : Load the Tux graphics and animations
 *******************************/
-static void LoadTuxAnims(void)
+static int LoadTuxAnims(void)
 {
-	int i;
-	int height = 0;                //temp width/height varis to determine max's
+  int i;
+  int height = 0;                //temp width/height varis to determine max's
 
-	LOG( "=LoadTuxAnims(): Loading Tux Animations\n" );
+  LOG("LoadTuxAnims(): Loading Tux Animations\n");
 
-	for ( i=0; i < TUX_NUM_STATES; i++ ) {
-		tux_object.spr[i][RIGHT] = LoadSprite( tux_sprite_fns[i], IMG_COLORKEY ); 
-		tux_object.spr[i][LEFT] = FlipSprite( tux_object.spr[i][RIGHT], 1, 0 ); 
-	}
+  for (i = 0 ; i < TUX_NUM_STATES; i++)
+  {
+    tux_object.spr[i][RIGHT] = LoadSprite(tux_sprite_fns[i], IMG_ALPHA); 
+    /* make sure image got loaded: */
+    if(tux_object.spr[i][RIGHT] == NULL)
+    {
+      fprintf(stderr, "Warning - image %d failed to load\n", i);
+      return 0;
+    }
+    tux_object.spr[i][LEFT] = FlipSprite(tux_object.spr[i][RIGHT], 1, 0); 
+  }
 
-	tux_max_width = tux_object.spr[TUX_STANDING][RIGHT]->frame[0]->w;
-	height        = tux_object.spr[TUX_STANDING][RIGHT]->frame[0]->h;
+  tux_max_width = tux_object.spr[TUX_STANDING][RIGHT]->frame[0]->w;
+  height = tux_object.spr[TUX_STANDING][RIGHT]->frame[0]->h;
 
-	LOG( "=LoadTuxAnims(): END\n" );
+  LOG("LoadTuxAnims(): END\n");
+
+  return 1;
 }
 
 /******************************
@@ -938,7 +979,7 @@ in the key cascade game
 *****************************/
 static void SpawnFishies(int diflevel, int* fishies, int* frame)
 {
-  int i, spacing;
+  int i, spacing, max_length;
   wchar_t* new_word;
 
   LOG("Enter SpawnFishies()\n");
@@ -989,15 +1030,24 @@ static void SpawnFishies(int diflevel, int* fishies, int* frame)
       return;
   }
 
-  /* See if we get a valid word before we move on: */
-  new_word = GetWord();
+  /* See how long of a word will fit the length of our screen: */
+  max_length = screen->w / fish_sprite->frame[0]->w;
 
+  do
+  {
+    new_word = GetWord();
+    if (!new_word) break;
+  }
+  while(wcslen(new_word) > max_length);
+
+  /* See if we get a valid word before we move on: */
   if (!new_word)
   {
     fprintf(stderr, "SpawnFishies() - could not get word - returning.\n");
     return;
   }
 
+
   /* If we get to here, it should be OK to actually spawn the fishy: */
   fish_object[*fishies].word = new_word;
   fish_object[*fishies].len = wcslen(new_word); //using wchar_t[] now
@@ -1163,6 +1213,17 @@ static void AddSplat(int* splats, struct fishypoo* f, int* curlives, int* frame)
 }
 
 
+#define NUM_RTL_LANGUAGES 4
+static int RTL()
+{
+  int i;
+  char* languages[NUM_RTL_LANGUAGES] = {"he_IL.utf8", "ar_AE.utf8", "ar_BH.utf8", "ar_DZ.utf8"}; //etc
+  for(i = 0; i< NUM_RTL_LANGUAGES; i++)
+    if (strcmp(settings.theme_locale_name, languages[i])==0)
+      return 1;
+  return 0;
+}
+
 
 
 static void DrawFish(int which)
@@ -1226,9 +1287,11 @@ static void DrawFish(int which)
         j++;
     }
 
+
     LOG ("Now draw each letter:\n");
 
-    for (j = 0; j < wcslen(fish_object[which].word); j++)
+    int length = wcslen(fish_object[which].word);
+    for (j = 0; j < length; j++)
     {
       current_letter = (int)fish_object[which].word[j];
 
@@ -1239,6 +1302,10 @@ static void DrawFish(int which)
 
       /* Set "letter_x" and "letter_y to where we want the letter drawn: */
       letter_x = fish_object[which].x + (j * fish_sprite->frame[0]->w) + x_inset;
+
+      if (RTL())
+	letter_x = fish_object[which].x + ((length-1-j) * fish_sprite->frame[0]->w) + x_inset;
+
       letter_y = fish_object[which].y + y_inset;
 
       if(letter_surface != NULL)
@@ -1520,6 +1587,72 @@ static void draw_bar(int curlevel, int diflevel, int curlives, int oldlives, int
 }
 
 
+/* Here we process the latest keystroke with the input_methods system. */ 
+/* Depending on whether the current language uses a single keystroke   */
+/* or multiple keystrokes per letter, we may or may not change what is displayed. */
+static void HandleKey(SDL_keysym* key_sym)
+{
+  static int redraw = 0;
+  wchar_t* im_cp = im_data.s;
+  int texttool_len = 0;
+  wchar_t texttool_str[256];
+  
+  /* Discard previous # of redraw characters */
+  if((int)texttool_len <= redraw) 
+    texttool_len = 0;
+  else texttool_len -= redraw;
+    texttool_str[texttool_len] = L'\0';
+
+  /* Read IM, remember how many to redraw next iteration */
+  redraw = im_read(&im_data, *key_sym);
+
+  /* Queue each character to be displayed */
+  while(*im_cp)
+  {
+    if (*im_cp == L'\b')
+    {
+      if (texttool_len > 0)
+      {
+        texttool_len--;
+        texttool_str[texttool_len] = 0;
+//        do_render_cur_text(0);
+      }
+    }
+    else if (*im_cp == L'\r')
+    {
+      if (texttool_len > 0)
+      {
+//        rec_undo_buffer();
+//        do_render_cur_text(1);
+        texttool_len = 0;
+      }
+      im_softreset(&im_data);
+    }
+    else if (*im_cp == L'\t')
+    {
+      if (texttool_len > 0)
+      {
+//        rec_undo_buffer();
+//        do_render_cur_text(1);
+//        cursor_x = min(cursor_x + cursor_textwidth, canvas->w);
+        texttool_len = 0;
+      }
+      im_softreset(&im_data);
+    }
+    //FIXME look up iswprint()
+    else if (1) //(iswprint(*im_cp))
+    {
+      if (texttool_len < (sizeof(texttool_str) / sizeof(wchar_t)) - 1)
+      {
+        texttool_str[texttool_len++] = *im_cp;
+        texttool_str[texttool_len] = 0;
+//        do_render_cur_text(0);
+      }
+    }
+
+    im_cp++;
+  } /* while(*im_cp) */
+}
 
 
 
