@@ -397,7 +397,10 @@ int Phrases(wchar_t* pphrase )
         switch(event.key.keysym.sym)
         {
           case  SDLK_ESCAPE:
-            quit = 1;
+            if (Pause() == 1)
+               quit = 1;
+            // continue loop and/or redraw screen
+            state = 1;
             break;
 
           case  SDLK_F10:
@@ -611,7 +614,8 @@ int Phrases(wchar_t* pphrase )
         if(shift_pressed)
           tmp=toupper(tmp);
 
-        updatekeylist(key,tmp);
+        if ( key != -1 ) 
+          updatekeylist(key,tmp);
 
         /* Record elapsed time for this keypress and update running total: */
         a = SDL_GetTicks();
@@ -619,7 +623,7 @@ int Phrases(wchar_t* pphrase )
         start = a;
 
         total += keytimes[cursor];
-        sprintf(time_str, "%.2f sec", (float) total / 1000);
+        sprintf(time_str, "%.2f %s", (float) total / 1000, N_("sec"));
         sprintf(chars_typed_str, "%d", correct_chars);
         sprintf(cpm_str, "%.1f", (float) correct_chars /((float)total/60000));
         sprintf(wpm_str, "%.1f", (float) ((float) correct_chars/5) /((float) total/60000));
@@ -777,13 +781,16 @@ int Phrases(wchar_t* pphrase )
         }
         else  /* -------- handle incorrect key press: -------------*/
         {
-          int key = GetIndex((wchar_t)event.key.keysym.unicode);
-          keypress1= GetWrongKeypress(key);
-
-          if (keypress1) // avoid segfault if NULL
+          // int key = GetIndex((wchar_t)event.key.keysym.unicode);
+          if ( key != -1 ) 
           {
-            SDL_BlitSurface(keypress1, NULL, screen, &keyboard_loc);
-            SDL_FreeSurface(keypress1);
+            keypress1= GetWrongKeypress(key);
+          
+            if (keypress1) // avoid segfault if NULL
+            {
+              SDL_BlitSurface(keypress1, NULL, screen, &keyboard_loc);
+              SDL_FreeSurface(keypress1);
+            }
           }
           state = 2;
 
@@ -874,8 +881,8 @@ static int practice_load_media(void)
   }
 
   /* load tux sprites: */
-  tux_win = LoadSprite("tux/win", IMG_COLORKEY);
-  tux_stand = LoadSprite("tux/stand", IMG_COLORKEY);
+  tux_win = LoadSprite("tux/win", IMG_ALPHA);
+  tux_stand = LoadSprite("tux/stand", IMG_ALPHA);
   /* load needed sounds: */
   wrong = LoadSound("buzz.wav");
   cheer = LoadSound("cheer.wav");
@@ -1461,4 +1468,4 @@ static int create_labels(void)
     return 1;
   else
     return 0;
-}
\ No newline at end of file
+}
