@@ -1,25 +1,37 @@
-/***************************************************************************
- -  file: scripting.c
- -  description: scripting for lessons & instructions ...
-                             -------------------
-    begin                : Sun Dec 28, 2003
-    copyright            : Jesse Andrews (C) 2003
-    email                : tuxtype-dev@tux4kids.net
-***************************************************************************/
+/*
+   scripting.c:
+
+   XML-based scripting for lessons and practice.
+   Copyright 2003, 2004, 2007, 2008, 2009, 2010.
+   Authors: Jesse Andrews, David Bruce, Matthew Trey.
+   
+   Project email: <tux4kids-tuxtype-dev@lists.alioth.debian.org>
+   Project website: http://tux4kids.alioth.debian.org
+
+   scripting.c is part of Tux Typing, a.k.a "tuxtype".
+
+Tux Typing is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+Tux Typing is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+
 
-/***************************************************************************
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License as published by  *
-*   the Free Software Foundation; either version 2 of the License, or     *
-*   (at your option) any later version.                                   *
-*                                                                         *
-***************************************************************************/
 
 #include "scripting.h"
 #define MAX_LESSONS 100
 #include "SDL_extras.h"
 #include "convert_utf.h"
+#include "scandir.h"
 
 /* Local function prototypes: */
 static void clear_items(itemType* i);
@@ -31,7 +43,7 @@ static char* get_quote(const char* in);
 static char hex2int(char b, char s);
 static int load_script(const char* fn);
 static void run_script(void);
-
+static int is_xml_file(const struct dirent* xml_dirent);
 /************************************************************************/
 /*                                                                      */ 
 /*         "Public" functions (callable throughout program)             */
@@ -65,7 +77,6 @@ void InstructCascade(void)
 void InstructLaser(void)
 {
   char fn[FNLEN]; 
-  int i;
 
   /* Try theme script first: */
   if (!settings.use_english)
@@ -73,11 +84,7 @@ void InstructLaser(void)
 
   if (load_script( fn ) == 0) /* meaning successful load */
   {
-    for (i = 0; i < 20; i++)
-    {
-      run_script();
-      SDL_Delay(500);
-    }
+    run_script();
     return;
   }
 
@@ -87,14 +94,9 @@ void InstructLaser(void)
   if (load_script( fn ) != 0)
     return; // bail if any errors occur
 
-  for (i = 0; i < 20; i++)
-  {
-    run_script();
-    SDL_Delay(500);
-  }
+  run_script();
 }
 
-
 void ProjectInfo(void)
 {
   char fn[FNLEN]; 
@@ -117,9 +119,12 @@ void ProjectInfo(void)
   run_script();
 }
 
-/* NOTE - confusing function name - this is the function that allows the user */
-/* to select a lesson from the menu.                                          */
-int TestLesson(void)
+/* This is the function that allows the user */
+/* to select a lesson from the menu.         */
+/* FIXME we ought to display descriptive titles from the lessons, */
+/* rather than just the filenames.  We also should consider a     */
+/* "gold stars" system like in TuxMath - DSB                      */
+int XMLLesson(void)
 {
   SDL_Surface* titles[MAX_LESSONS] = {NULL};
   SDL_Surface* select[MAX_LESSONS] = {NULL};
@@ -127,20 +132,23 @@ int TestLesson(void)
   SDL_Rect leftRect, rightRect;
   SDL_Rect titleRects[8];
 
+  int nchars;
+  struct dirent **script_list_dirents = NULL;
+  int i = 0;
+  int scriptIterator = 0;  //Iterator over matching files in script dir
+  int scripts = 0;         //Iterator over accepted (& parsed) script files
+  int num_scripts = 0;
+  char script_path[FNLEN];
+  char script_filenames[MAX_LESSONS][FNLEN];
+  char fn[FNLEN]; 
+
   int stop = 0;
   int loc = 0;
   int old_loc = 1;
-  int num_scripts = 0;
   int found = 0;
-  int i;
-  char script_path[FNLEN];
-  char script_filenames[MAX_LESSONS][200];
-  char fn[FNLEN]; 
 
-  DIR* script_dir = NULL;
-  struct dirent* script_file = NULL;
 
-  LOG("Entering TestLesson():\n");
+  LOG("Entering XMLLesson():\n");
 
   /* First look in theme path, if desired: */
   if (!settings.use_english)
@@ -166,7 +174,7 @@ int TestLesson(void)
 
   if (!found)
   {
-    fprintf(stderr, "TestLesson(): Error finding script directory!\n");
+    fprintf(stderr, "XMLLesson(): Error finding script directory!\n");
     return 0;
   }
 
@@ -177,36 +185,92 @@ int TestLesson(void)
   DEBUGCODE { fprintf(stderr, "script_path is: %s\n", script_path); }
 
 
-  /* FIXME looks like a place for scandir() - or our own w32_scandir() */
   /* create a list of all the .xml files */
+  num_scripts = scandir(script_path, &script_list_dirents, is_xml_file, alphasort);
 
-  script_dir = opendir(script_path);	
-
-  do
+  for (scriptIterator = 0, scripts = 0;
+       scriptIterator < num_scripts && scripts < MAX_LESSONS;
+       scriptIterator++)
   {
-    script_file = readdir(script_dir);
-    if (!script_file)
-      break;
+    /* Copy over the filename: */
+    nchars = snprintf(script_filenames[scripts], FNLEN, "%s",
+                      script_list_dirents[scriptIterator]->d_name);
 
-    /* must have at least '.xml' at the end */
-    if (strlen(script_file->d_name) < 5)
+    /* Skip (actually clobber) any invalid or undesired files: */
+    if (nchars < 0 || nchars >= FNLEN)
+      continue;
+    /* Don't show project info file or instructions files */
+    if (strcmp(script_filenames[scripts], "projectInfo.xml") == 0 ||
+        strcmp(script_filenames[scripts], "laser.xml") == 0 ||
+        strcmp(script_filenames[scripts], "cascade.xml") == 0)
       continue;
 
-    /* Don't show project info file: */
-    if (strcmp(script_file->d_name, "projectInfo.xml") == 0)
-      continue;
+    DEBUGCODE
+    {
+      fprintf(stderr, "Found script file %d:\t%s\n", scripts, script_filenames[scripts]);
+    }
 
-    if (strcmp(&script_file->d_name[strlen(script_file->d_name) - 4],".xml"))
-      continue;
+    /* Increment the iterator for correctly-parsed lesson files */
+    scripts++;
+  }
 
-    sprintf(script_filenames[num_scripts], "%s", script_file->d_name);
-    num_scripts++;
-    DEBUGCODE { fprintf(stderr, "Adding XML file no. %d: %s\n",
-                num_scripts, script_filenames[num_scripts]); }
+//  DEBUGCODE  
+  {
+    fprintf(stderr, "Before undesired files screened out:\n");
+    for(i = 0; i < num_scripts; i++)
+      fprintf(stderr, "script %d filename: %s\n", i,
+              script_list_dirents[i]->d_name);
+    fprintf(stderr, "After undesired files screened out:\n");
+    for(i = 0; i < scripts; i++)
+      fprintf(stderr, "script %d filename: %s\n", i,
+              script_filenames[i]);
+  }
+
+
+  /* Now free the individual dirents. We do this on a second pass */
+  /* because of the "continue" approach used to error handling.   */
+  for (scriptIterator = 0; scriptIterator < num_scripts; scriptIterator++)
+    free(script_list_dirents[scriptIterator]);
+  free(script_list_dirents);
+
+  /* Adjust num_scripts for any skipped files: */
+  num_scripts = scripts;
+
+//START OF OLD IMPLEMENTATION
+//   num_scripts = 0;
+//   script_dir = opendir(script_path);	
+//   do
+//   {
+//     script_file = readdir(script_dir);
+//     if (!script_file)
+//       break;
+// 
+//     /* must have at least '.xml' at the end */
+//     if (strlen(script_file->d_name) < 5)
+//       continue;
+// 
+//     /* Don't show project info file or instructions files */
+//     if (strcmp(script_file->d_name, "projectInfo.xml") == 0 ||
+//         strcmp(script_file->d_name, "laser.xml") == 0 ||
+//         strcmp(script_file->d_name, "cascade.xml") == 0)
+//       continue;
+// 
+// 
+//     if (strcmp(&script_file->d_name[strlen(script_file->d_name) - 4],".xml"))
+//       continue;
+// 
+//     sprintf(script_filenames[num_scripts], "%s", script_file->d_name);
+//     num_scripts++;
+//     DEBUGCODE { fprintf(stderr, "Adding XML file no. %d: %s\n",
+//                 num_scripts, script_filenames[num_scripts]); }
+// 
+//   } while (1); /* Leave loop when readdir() returns NULL */
+// 
+//   closedir(script_dir);	
+
+// END OF OLD IMPLEMENTATION
 
-  } while (1); /* Leave loop when readdir() returns NULL */
 
-  closedir(script_dir);	
 
   DEBUGCODE { fprintf(stderr, "Found %d . xml file(s) in script dir\n", num_scripts); }
 
@@ -225,7 +289,7 @@ int TestLesson(void)
   /* Get out if needed surface not loaded successfully: */
   if (!CurrentBkgd() || !left || !right)
   {
-    fprintf(stderr, "TestLesson(): needed image not available\n");
+    fprintf(stderr, "XMLLesson(): needed image not available\n");
   
     for (i = 0; i < num_scripts; i++)
     {
@@ -245,10 +309,12 @@ int TestLesson(void)
   leftRect.w = left->w;
   leftRect.h = left->h;
   leftRect.x = screen->w/2 - 80 - (leftRect.w/2);
-  leftRect.y = screen->h/2 - 50;
+  leftRect.y = screen->h - 50;
 
-  rightRect.w = right->w; rightRect.h = right->h;
-  rightRect.x = screen->w/2 + 80 - (rightRect.w/2); rightRect.y = screen->h/2 - 50;
+  rightRect.w = right->w; 
+  rightRect.h = right->h;
+  rightRect.x = screen->w/2 + 80 - (rightRect.w/2);
+  rightRect.y = screen->h - 50;
 
   /* set initial rect sizes */
   titleRects[0].y = 30;
@@ -268,7 +334,7 @@ int TestLesson(void)
       switch (event.type)
       {
         case SDL_QUIT:
-          exit(0); /* FIXME may need to cleanup memory and exit more cleanly */
+          return 0; /* Return control to the main program so we can exit cleanly */
           break;
 
         case SDL_MOUSEMOTION:
@@ -304,7 +370,10 @@ int TestLesson(void)
             if (inRect(titleRects[i], event.button.x, event.button.y))
             {
               loc = loc - (loc % 8) + i;
-              sprintf(fn, "%s/scripts/%s", settings.default_data_path, script_filenames[loc]);
+              if(settings.use_english)
+                sprintf(fn, "%s/scripts/%s", settings.default_data_path, script_filenames[loc]);
+              else
+                sprintf(fn, "%s/scripts/%s", settings.theme_data_path, script_filenames[loc]);
               stop = 1;
               break; 
             }
@@ -321,8 +390,10 @@ int TestLesson(void)
 
           if (event.key.keysym.sym == SDLK_RETURN)
           {
-            sprintf(fn, "%s/scripts/%s", settings.default_data_path, script_filenames[loc]);
-            stop = 1;
+            if(settings.use_english)
+              sprintf(fn, "%s/scripts/%s", settings.default_data_path, script_filenames[loc]);
+            else
+              sprintf(fn, "%s/scripts/%s", settings.theme_data_path, script_filenames[loc]);            stop = 1;
             break;
           }
 
@@ -340,13 +411,17 @@ int TestLesson(void)
               loc = (loc - (loc % 8) + 8);
           }
 
-          if (event.key.keysym.sym == SDLK_UP)
+          if ((event.key.keysym.sym == SDLK_UP)
+	     ||
+	      (event.key.keysym.sym == SDLK_k))
           {
             if (loc > 0)
               loc--;
           }
 
-          if (event.key.keysym.sym == SDLK_DOWN)
+          if ((event.key.keysym.sym == SDLK_DOWN)
+	     ||
+	      (event.key.keysym.sym == SDLK_j))
           {
             if (loc + 1 < num_scripts)
               loc++;
@@ -422,7 +497,7 @@ int TestLesson(void)
   run_script();
   SDL_ShowCursor(1);
 
-  LOG("Leave TestLesson()\n");
+  LOG("Leave XMLLesson()\n");
 
   return 1;
 }
@@ -542,6 +617,72 @@ static int load_script(const char* fn)
     if (fscanf_result == EOF)
       break;
 
+    if (strncmp("<!--", str, 4) == 0)
+    {
+        /* -- comment section found, ignore everything until comment close -- */
+        int found = 0;
+        char* tmpStr;
+
+        do
+        {
+            // search the current line for comment end
+            for ( tmpStr = str; strlen(tmpStr) >= 3 && !found; tmpStr++ )
+            {    
+                 if (strncmp("-->",tmpStr, 3) == 0)
+                 {
+                     // move past the comment end tag
+                     tmpStr += 2;
+                     found = 1;
+                 }
+            }
+
+            // if the comment end was not found get another line
+            if (!found)
+            {
+                fscanf_result = fscanf(f, "%[^\n]\n", str);
+                tmpStr = str;
+            }
+
+            // we did find the end of the comment
+            else
+            {
+ 
+                if (strlen(tmpStr) > 0)
+                {
+                    // copy the rest of the line into str for processing
+                    strncpy(str, tmpStr, strlen(tmpStr));
+                    str[strlen(tmpStr)] = '\0';
+                }
+                else
+                {
+                    // str needs another line, this one is used up
+                    fscanf_result = fscanf(f, "%[^\n]\n", str);
+                    tmpStr = str;
+                }
+                
+                // if the next line is a comment, start all over again
+                if (fscanf_result != EOF && strncmp("<!--", str, 4) == 0)
+                {
+                    found = 0;
+                }
+            }
+
+        } while ( fscanf_result != EOF && !found );
+        
+        /* -- if we reached the end of the file and saw no close to the comment, generate a warning -- */
+        if ( !found && fscanf_result == EOF )
+        {
+            fprintf(stderr, "XML Warning: End of file reached looking for the end of a comment.\n");
+            break;
+        }
+
+        /* -- don't continue processing if at EOF -- */
+        if (fscanf_result == EOF)
+        {
+            break;
+        }
+    }
+
     if (strncmp("<script", str, 7) == 0)
     {
       /* -- allocate space for the lesson info -- */
@@ -565,7 +706,9 @@ static int load_script(const char* fn)
     {
       if (curScript==NULL)
       {
-        fprintf(stderr, "CRITICAL XML ERROR: <page> should be in a <script> in file %s line (todo)", fn); exit(1);
+        fprintf(stderr, "CRITICAL XML ERROR: <page> should be in a <script> in file %s line (todo)", fn);
+        close_script();
+        return 0;
       }
 
       if (curScript->pages==NULL)
@@ -600,7 +743,9 @@ static int load_script(const char* fn)
     {
       if (curPage == NULL)
       {
-        fprintf(stderr, "CRITICAL XML ERROR: <text> should be in a <page> in file %s line (todo)", fn); exit(1);
+        fprintf(stderr, "CRITICAL XML ERROR: <text> should be in a <page> in file %s line (todo)", fn);
+        close_script();
+        return 0;
       }
 
       if (curPage->items == NULL) 
@@ -687,7 +832,8 @@ static int load_script(const char* fn)
         fprintf(stderr,
                 "CRITICAL XML ERROR: <img> should be in a <page> in file %s line (todo)",
                fn);
-        exit(1); //FIXME should call cleanup() or maybe just return without exiting
+        close_script();
+        return 0; //Return control to the main program for a clean exit
       }
 
       if (curPage->items == NULL)
@@ -740,6 +886,39 @@ static int load_script(const char* fn)
         }
       }
     }
+    else if (strncmp("<bkgd", str, 5) == 0)
+    {
+      if (curPage == NULL)
+      {
+        fprintf(stderr,
+                "CRITICAL XML ERROR: <bkgd> should be in a <page> in file %s line (todo)",
+               fn);
+        close_script();
+        return 0; //Return control to the main program for a clean exit
+      }
+
+      if (curPage->items == NULL)
+      {
+        curItem = (itemType*)calloc(1, sizeof(itemType));
+        curPage->items = curItem;
+      }
+      else 
+      {
+        curItem->next = (itemType*)calloc(1, sizeof(itemType));
+        curItem = curItem->next;
+      }
+
+      curItem->type = itemBKGD;
+      curItem->x = curItem->y = -1;
+
+
+      for (i = 6; i < strlen(str); i++) 
+      {
+        if ((str[i] == 's') && strncmp("src", &str[i], 3) == 0)
+          curItem->data = get_quote(&str[i + 3]);
+      }
+    }
+
     else if (strncmp("<wav", str, 4) == 0)
     {
       if (curPage == NULL)
@@ -747,7 +926,8 @@ static int load_script(const char* fn)
         fprintf(stderr,
                 "CRITICAL XML ERROR: <wav> should be in a <page> in file %s line (todo)",
                 fn);
-        exit(1);  /* FIXME call Cleanup() to exit */
+        close_script();
+        return 0;  /* Return control to main program for a clean exit */
       }
 
       if (curPage->items == NULL)
@@ -788,8 +968,10 @@ static int load_script(const char* fn)
       {
         fprintf(stderr,
                 "CRITICAL XML ERROR: <prac> should be in a <page> in file %s line (todo)",
+
                 fn);
-        exit(1); /* FIXME call cleanup() rather than exit() */
+        close_script();
+        return 0; /* Return control to the main program for a clean exit */
       }
 
       if (curPage->items == NULL)
@@ -870,7 +1052,8 @@ static int load_script(const char* fn)
         fprintf(stderr,
                "CRITICAL XML ERROR: <waitforinput> should be in a <page> in file %s line (todo)",
                 fn);
-        exit(1);
+        close_script();
+        return 0;
       }
 
       if (curPage->items == NULL)
@@ -894,7 +1077,8 @@ static int load_script(const char* fn)
         fprintf(stderr,
                "CRITICAL XML ERROR: <waitforchar> should be in a <page> in file %s line (todo)",
                fn);
-        exit(1);
+        close_script();
+        return 0;
       }
 
       if (curPage->items == NULL)
@@ -927,6 +1111,8 @@ static int load_script(const char* fn)
 }
 
 
+
+
 static void run_script(void)
 {
   /* FIXME FNLEN doesn't make sense for size of these arrays */
@@ -965,13 +1151,39 @@ static void run_script(void)
     if (curPage->background)
     {
       SDL_Surface* img = LoadImage(curPage->background, IMG_ALPHA|IMG_NOT_REQUIRED);
-      SDL_BlitSurface(img, NULL, screen, NULL);
+
+      /* hack: since this is the background it needs to scale when in fullscreen
+       * but shouldn't every image scale when in fullscreen? assuming svg is for that... -MDT */
+      if (settings.fullscreen)
+      {
+        SDL_Surface* fsimg = zoom(img, fs_res_x, fs_res_y);
+        SDL_BlitSurface(fsimg, NULL, screen, NULL);
+        SDL_FreeSurface(fsimg);
+      }
+      else
+      { 
+        SDL_BlitSurface(img, NULL, screen, NULL);
+      } 
+
       SDL_FreeSurface(img);
+
     }
     else if (curScript->background)
     {
       SDL_Surface* img = LoadImage(curScript->background, IMG_ALPHA|IMG_NOT_REQUIRED);
-      SDL_BlitSurface(img, NULL, screen, NULL);
+
+      /* hack: since this is the background it needs to scale when in fullscreen -MDT */
+      if (settings.fullscreen)
+      { 
+        SDL_Surface* fsimg = zoom(img, fs_res_x, fs_res_y);
+        SDL_BlitSurface(fsimg, NULL, screen, NULL);
+        SDL_FreeSurface(fsimg);
+      }
+      else
+      { 
+        SDL_BlitSurface(img, NULL, screen, NULL);
+      } 
+
       SDL_FreeSurface(img);
     }
 
@@ -1036,8 +1248,30 @@ static void run_script(void)
               clickRects[numClicks].h = loc.h;
               numClicks++;
             }
+
+            SDL_FreeSurface(img);
+          }
+          break;
+        }
+
+        case itemBKGD:
+        {
+          SDL_Surface* img = LoadImage(curItem->data, IMG_ALPHA|IMG_NOT_REQUIRED);
+          if (img)
+          {
+            /* hack: since this is the background it needs to scale when in fullscreen -MDT */
+            if (settings.fullscreen)
+            {
+              SDL_Surface* fsimg = zoom(img, fs_res_x, fs_res_y);
+              SDL_BlitSurface(fsimg, NULL, screen, NULL);
+              SDL_FreeSurface(fsimg);
+            }
+            else
+            {
+              SDL_BlitSurface(img, NULL, screen, NULL);
+            }
+            SDL_FreeSurface(img);
           }
-          SDL_FreeSurface(img);
           break;
         }
 
@@ -1081,7 +1315,9 @@ static void run_script(void)
                 continue;
 
               tmp[toshow] = 0;
+ 
               img = SimpleText(tmp, (int)curItem->size, col);
+
               if (img)
               { 
                 if (img->w + 20 < screen->w)
@@ -1142,7 +1378,11 @@ static void run_script(void)
         {
           // HACK, we need to make sure no more than 8 sounds or so..
           sounds[numWavs] = LoadSound( curItem->data );
-          Mix_PlayChannel( numWavs, sounds[numWavs], -curItem->loop );
+
+          // let audio.c handle calls to SDL_mixer
+          //Mix_PlayChannel( numWavs, sounds[numWavs], -curItem->loop );
+          
+          PlaySoundLoop( sounds[numWavs], -curItem->loop );
           numWavs++;
           break;
         }
@@ -1170,7 +1410,9 @@ static void run_script(void)
                   for (j=0; j<numClicks; j++) 
                   {
                     if (inRect(clickRects[j], event.button.x, event.button.y))
-                     Mix_PlayChannel(numWavs + j, clickWavs[j], 0);
+                     PlaySound( clickWavs[j] );
+                     // let audio.c handle calls to SDL_mixer
+                     //Mix_PlayChannel(numWavs + j, clickWavs[j], 0);
                   }
                   break;
                 }
@@ -1306,15 +1548,22 @@ static void run_script(void)
 
       if (settings.sys_sound)
       {
+        // halt all the channels before we try to free the sounds
+        audioHaltChannel(-1);
+
         for (i=0; i<numWavs; i++)
         {
-          Mix_HaltChannel(i);
+          // let audio.c handle calls to SDL_mixer
+          //Mix_HaltChannel(i);
+
           Mix_FreeChunk(sounds[i]);
         }
 
         for (i = 0; i < numClicks; i++)
         {
-          Mix_HaltChannel(i + numWavs);
+          // let audio.c handle calls to SDL_mixer
+          //Mix_HaltChannel(i + numWavs);
+
           Mix_FreeChunk(clickWavs[i]);
         }
       }
@@ -1395,5 +1644,10 @@ static void close_script(void)
 }
 
 
-
-
+/* NOTE we just check to see if the name ends in ".xml", but we don't */
+/* verify that it really contains XML.                                */
+static int is_xml_file(const struct dirent* xml_dirent)
+{
+  const char* ending = &xml_dirent->d_name[strlen(xml_dirent->d_name) - 4]; 
+  return (0 == strncasecmp(ending, ".xml", 4));
+}
