@@ -1,27 +1,38 @@
-/***************************************************************************
- -  file: titlescreen.c
- -  description: splash, title and menu screen functionality 
-                            ------------------
-    begin                : Thur May 4 2000
-    copyright            : (C) 2000 by Sam Hart
-                         : (C) 2003 by Jesse Andrews
-    email                : tuxtype-dev@tux4kids.net
-***************************************************************************/
+/*
+   titlescreen.c:
+
+   Splash screen and menu code.
+   
+   Copyright 2003, 2004, 2007, 2008, 2009, 2010.
+   Authors: Jesse Andrews, David Bruce.
+   
+   Project email: <tux4kids-tuxtype-dev@lists.alioth.debian.org>
+   Project website: http://tux4kids.alioth.debian.org
+
+   titlescreen.c is part of Tux Typing, a.k.a "tuxtype".
+
+Tux Typing is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+Tux Typing is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
 
 #include "globals.h"
 #include "funcs.h"
 #include "titlescreen.h"
 #include "SDL_extras.h"
 #include "convert_utf.h"
+#include "editor.h"
 
 /* --- media for menus --- */
 
@@ -58,8 +69,6 @@ static int load_media(void);
 static void load_menu(void);
 static void recalc_rects(void);
 static int chooseWordlist(void);
-static void ChooseWord(char* words_file);
-static void ChooseFile(void);
 static void unload_media(void);
 static void unload_menu(void);
 static void not_implemented(void);
@@ -87,11 +96,11 @@ const char* menu_icon[][6] =
      
 static const char* menu_text[] = 
 {"", "",            "",             "",            ""    ,
- "",gettext_noop("Fish Cascade"),gettext_noop("Easy"),gettext_noop("Space Cadet"),gettext_noop("Edit Word Lists"),
- "",gettext_noop("Comet Zap"),gettext_noop("Medium"),gettext_noop("Pilot"),gettext_noop("Phrase Typing"),
- "",gettext_noop("Lessons"),gettext_noop("Hard"), gettext_noop("Ace"),gettext_noop("Project Info"),
- "", gettext_noop("Options"),gettext_noop("Instructions"),gettext_noop("Commander"),gettext_noop("Setup Language"),
- "", gettext_noop("Quit"),gettext_noop("Main Menu"),gettext_noop("Main Menu"),gettext_noop("Main Menu")};
+ "",N_("Fish Cascade"),N_("Easy"),N_("Space Cadet"),N_("Edit Word Lists"),
+ "",N_("Comet Zap"),N_("Medium"),N_("Pilot"),N_("Phrase Typing"),
+ "",N_("Lessons"),N_("Hard"), N_("Ace"),N_("Project Info"),
+ "", N_("Options"),N_("Instructions"),N_("Commander"),N_("Setup Language"),
+ "", N_("Quit"),N_("Main Menu"),N_("Main Menu"),N_("Main Menu")};
 
 
 
@@ -257,7 +266,7 @@ void TitleScreen(void)
   while (!done) 
   {
 
-    start=SDL_GetTicks();
+    start = SDL_GetTicks();
 
     /* ---process input queue --- */
 
@@ -270,11 +279,15 @@ void TitleScreen(void)
     {
       switch (event.type)
       {
-
+        /* Update "selection" if mouse moves within a menu entry: */
         case SDL_MOUSEMOTION:
         {
           cursor.x = event.motion.x;
           cursor.y = event.motion.y;
+
+	  for (j = 1; j <= TITLE_MENU_ITEMS; j++)
+            if (inRect(menu_button[j], cursor.x, cursor.y))
+              key_menu = j;
           break;
         }
 
@@ -388,6 +401,7 @@ void TitleScreen(void)
 
 
             case SDLK_UP:
+	    case SDLK_k:
             {
               if (settings.menu_sound)
                 PlaySound(snd_move);
@@ -399,6 +413,7 @@ void TitleScreen(void)
 
 
             case SDLK_DOWN:
+	    case SDLK_j:
             {
               key_menu++;
               if (settings.menu_sound)
@@ -484,8 +499,7 @@ void TitleScreen(void)
 
     if (menu_opt == EDIT_WORDLIST)
     {
-      not_implemented();
-//      ChooseFile();
+      ChooseListToEdit();
       redraw = 1;
     }
 
@@ -501,13 +515,13 @@ void TitleScreen(void)
     {
 //      not_implemented();
       SDL_BlitSurface(CurrentBkgd(), NULL, screen, NULL);
-      SDL_Flip( screen );
+      SDL_Flip(screen);
       unload_media();
 
       if (settings.menu_music)
         MusicUnload( );
 
-      TestLesson();
+      XMLLesson();
 
       load_media();
 
@@ -624,19 +638,19 @@ void TitleScreen(void)
 
     if (menu_opt == INSTRUCT)
     {
-//      unload_media();
-      not_implemented();
+//       not_implemented();
+       unload_media();
 
-//       switch (sub_menu)
-//       {
-//         case CASCADE: InstructCascade(); break;
-//         case LASER:   InstructLaser();   break;
-//       }
+      switch (sub_menu)
+      {
+        case CASCADE: InstructCascade(); break;
+        case LASER:   InstructLaser();   break;
+      }
 
-//      load_media();
+      load_media();
 
-//      if (settings.menu_music)
-//        MusicLoad( "tuxi.ogg", -1 );
+      if (settings.menu_music)
+        MusicLoad( "tuxi.ogg", -1 );
 
       redraw = 1;
     }
@@ -754,8 +768,8 @@ void TitleScreen(void)
 
     /* --- check if mouse is in a menu option --- */
 
-    key_menu = 0;
-
+//    key_menu = 0;
+/*
     for (j = 1; j <= TITLE_MENU_ITEMS; j++)
     {
       if ((cursor.x >= menu_button[j].x && cursor.x <= (menu_button[j].x + menu_button[j].w)) &&
@@ -764,7 +778,7 @@ void TitleScreen(void)
         key_menu = j; // update menu to point
         break;        // Don't need to check rest of menu
       }
-    }
+    }*/
 
 
     /* --- return old selection to unselected state --- */
@@ -1112,9 +1126,9 @@ static void not_implemented(void)
 
   LOG( "NotImplemented() - creating text\n" );
 
-  s1 = BlackOutline( gettext_noop("Work In Progress!"), DEFAULT_MENU_FONT_SIZE, &white);
-  s2 = BlackOutline( gettext_noop("This feature is not ready yet"), DEFAULT_MENU_FONT_SIZE, &white);
-  s3 = BlackOutline( gettext_noop("Discuss the future of TuxTyping at"), DEFAULT_MENU_FONT_SIZE, &white);
+  s1 = BlackOutline( _("Work In Progress!"), DEFAULT_MENU_FONT_SIZE, &white);
+  s2 = BlackOutline( _("This feature is not ready yet"), DEFAULT_MENU_FONT_SIZE, &white);
+  s3 = BlackOutline( _("Discuss the future of TuxTyping at"), DEFAULT_MENU_FONT_SIZE, &white);
   s4 = BlackOutline( "http://tux4kids.alioth.debian.org/", DEFAULT_MENU_FONT_SIZE, &white);
 
   tux = LoadSprite("tux/tux-egypt", IMG_ALPHA);
@@ -1185,7 +1199,6 @@ static void not_implemented(void)
 
 
 
-#define MAX_WORD_LISTS 100
 
 /* returns 0 if user pressed escape ...
  *         1 if word list was set correctly
@@ -1215,43 +1228,46 @@ static int chooseWordlist(void)
 
   /* find the directory to load wordlists from */
 
-  /* Check under theme directory first, if theme selected: */
-  if (!settings.use_english)  /* Using theme: */
+  /* First we add the bundled word lists in either NLS or default theme: */
   {
-    sprintf(wordPath,"%s/words", settings.theme_data_path);
-    if (!CheckFile(wordPath))
+    if (!settings.use_english)  /* Using theme: */
     {
-      fprintf(stderr, "chooseWordList() - theme contains no wordlist dir \n");
-      return 0;
+      sprintf(wordPath,"%s/words", settings.theme_data_path);
+      if (!CheckFile(wordPath))
+      {
+        fprintf(stderr, "chooseWordList() - theme contains no wordlist dir \n");
+        return 0;
+      }
     }
-  }
-  else  /* No theme selected - using English: */
-  {
-    sprintf(wordPath,"%s/words", settings.default_data_path);
-    if (!CheckFile(wordPath))
+    else  /* No user settings or theme selected - using English: */
     {
-      fprintf(stderr, "chooseWordList() - data path contains no wordlist dir \n");
-      return 0;
+      sprintf(wordPath,"%s/words", settings.default_data_path);
+      if (!CheckFile(wordPath))
+      {
+        fprintf(stderr, "chooseWordList() - data path contains no wordlist dir \n");
+        return 0;
+      }
     }
   }
 
   /* If we get to here, we know there is at least a wordlist directory */
   /* but not necessarily any valid files.                              */
 
-  DEBUGCODE { fprintf(stderr, "wordPath is: %s\n", wordPath); }
+  DEBUGCODE { fprintf(stderr, "bundled wordPath is: %s\n", wordPath); }
 
 
   /* FIXME looks like a place for scandir() - or our own w32_scandir() */
+  /* FIXME we also have this block of code repeated three times - DSB  */
   /* create a list of all the .txt files */
-
   wordsDir = opendir( wordPath );	
-
   do
   {
     wordsFile = readdir(wordsDir);
     if (!wordsFile)
       break; /* Loop continues until break occurs */
 
+    DEBUGCODE { fprintf(stderr, "wordsFile name is: %s\n", wordsFile->d_name); }
+
     /* must have at least .txt at the end */
     if (strlen(wordsFile->d_name) < 5)
       continue;
@@ -1278,23 +1294,124 @@ static int chooseWordlist(void)
 
     fclose(tempFile);
   } while (1); /* Loop continues until break occurs */
-
   closedir(wordsDir);	
+
+  
+  
+  /* Adding global custom wordlists ------------------------------------ */
  
- DEBUGCODE { fprintf(stderr, "Found %d .txt file(s) in words dir\n", lists); }
+  sprintf(wordPath,"%s/words", settings.var_data_path);
+  if (!CheckFile(wordPath))
+  {
+    DEBUGCODE { fprintf(stderr, "chooseWordList() -  path \"%s\" not found\n", wordPath); }
+  }
+  else
+  {
+    /* If we get to here, we know there is at least a wordlist directory */
+    /* but not necessarily any valid files.                              */
 
+    DEBUGCODE { fprintf(stderr, "global custom wordPath is: %s\n", wordPath); }
 
+    /* FIXME looks like a place for scandir() - or our own w32_scandir() */
+    /* create a list of all the .txt files */
+    wordsDir = opendir( wordPath );	
+    do
+    {
+      wordsFile = readdir(wordsDir);
+      if (!wordsFile)
+        break; /* Loop continues until break occurs */
 
-  /* let the user pick the list */
+      DEBUGCODE { fprintf(stderr, "wordsFile name is: %s\n", wordsFile->d_name); }
+
+      /* must have at least .txt at the end */
+      if (strlen(wordsFile->d_name) < 5)
+        continue;
+
+      if (strcmp(&wordsFile->d_name[strlen(wordsFile->d_name) -4 ],".txt"))
+        continue;
+
+      sprintf(wordlistFile[lists], "%s/%s", wordPath, wordsFile->d_name);
+
+      /* load the name for the wordlist from the file ... (1st line) */
+      tempFile = fopen( wordlistFile[lists], "r" );
+      if (!tempFile)
+        continue;
+
+      result = fscanf(tempFile, "%[^\n]\n", wordlistName[lists]);
+      if (result == EOF)
+        continue;
+
+      /* check to see if it has a \r at the end of it (dos format!) */
+      if (wordlistName[lists][strlen(wordlistName[lists]) - 1] == '\r')
+        wordlistName[lists][strlen(wordlistName[lists]) - 1] = '\0';
+
+      lists++;
+
+      fclose(tempFile);
+    } while (1); /* Loop continues until break occurs */
+    closedir(wordsDir);
+  }
+  
+  /* Now add any lists in the user's personal settings path: ------------ */
+
+  sprintf(wordPath,"%s/words", settings.user_settings_path);
+  if (!CheckFile(wordPath))
+  {
+    DEBUGCODE { fprintf(stderr, "chooseWordList() -  path \"%s\" not found\n", wordPath); }
+  }
+  else
+  {
+    /* If we get to here, we know there is at least a wordlist directory */
+    /* but not necessarily any valid files.                              */
+
+    DEBUGCODE { fprintf(stderr, "user-specific wordPath is: %s\n", wordPath); }
+
+    /* FIXME looks like a place for scandir() - or our own w32_scandir() */
+    /* create a list of all the .txt files */
+    wordsDir = opendir( wordPath );	
+    do
+    {
+      wordsFile = readdir(wordsDir);
+      if (!wordsFile)
+        break; /* Loop continues until break occurs */
+
+      DEBUGCODE { fprintf(stderr, "wordsFile name is: %s\n", wordsFile->d_name); }
+
+      /* must have at least .txt at the end */
+      if (strlen(wordsFile->d_name) < 5)
+        continue;
+
+      if (strcmp(&wordsFile->d_name[strlen(wordsFile->d_name) -4 ],".txt"))
+        continue;
+
+      sprintf(wordlistFile[lists], "%s/%s", wordPath, wordsFile->d_name);
+
+      /* load the name for the wordlist from the file ... (1st line) */
+      tempFile = fopen( wordlistFile[lists], "r" );
+      if (!tempFile)
+        continue;
+
+      result = fscanf(tempFile, "%[^\n]\n", wordlistName[lists]);
+      if (result == EOF)
+        continue;
+
+      /* check to see if it has a \r at the end of it (dos format!) */
+      if (wordlistName[lists][strlen(wordlistName[lists]) - 1] == '\r')
+        wordlistName[lists][strlen(wordlistName[lists]) - 1] = '\0';
+
+      lists++;
+
+      fclose(tempFile);
+    } while (1); /* Loop continues until break occurs */
+    closedir(wordsDir);
+  }
+
+  DEBUGCODE { fprintf(stderr, "Found %d .txt file(s) in words dir\n", lists); }
+
+  
+  /* Done scanning for word lists, now display them for user selection: */
 
   /* Render SDL_Surfaces for list entries: */
-//  titles[0] = BlackOutline( _("Alphabet"), font, &white );
-//  select[0] = BlackOutline( _("Alphabet"), font, &yellow);
-
-  /* NOTE - no longer hard-coding titles[0] to be alphabet - themes  */
-  /* should include a regular word list file called "alphabet.txt"   */
-  /* Should sort the list and always put the alphabet file first, if */
-  /* present.                                                        */
   for (i = 0; i < lists; i++)
   {
     titles[i] = BlackOutline( wordlistName[i], DEFAULT_MENU_FONT_SIZE, &white );
@@ -1327,10 +1444,12 @@ static int chooseWordlist(void)
   leftRect.w = left->w;
   leftRect.h = left->h;
   leftRect.x = screen->w/2 - 80 - (leftRect.w/2);
-  leftRect.y = screen->h/2 - 50;
+  leftRect.y = screen->h - 50;
 
-  rightRect.w = right->w; rightRect.h = right->h;
-  rightRect.x = screen->w/2 + 80 - (rightRect.w/2); rightRect.y = screen->h/2 - 50;
+  rightRect.w = right->w;
+  rightRect.h = right->h;
+  rightRect.x = screen->w/2 + 80 - (rightRect.w/2);
+  rightRect.y = screen->h - 50;
 
   /* set initial rect sizes */
   titleRects[0].y = 30;
@@ -1424,13 +1543,17 @@ static int chooseWordlist(void)
               loc = (loc - (loc % 8) + 8);
           }
 
-          if (event.key.keysym.sym == SDLK_UP)
+          if ((event.key.keysym.sym == SDLK_UP)
+	     ||
+	      (event.key.keysym.sym == SDLK_k))
           {
             if (loc > 0)
               loc--;
           }
 
-          if (event.key.keysym.sym == SDLK_DOWN)
+          if ((event.key.keysym.sym == SDLK_DOWN)
+	     ||
+	      (event.key.keysym.sym == SDLK_j))
           {
             if (loc+1<lists)
               loc++;
@@ -1489,506 +1612,3 @@ static int chooseWordlist(void)
 
   return 1;
 }
-
-
-static void ChooseFile(void)
-{
-  SDL_Surface* titles[MAX_WORD_LISTS] = {NULL};
-  SDL_Surface* select[MAX_WORD_LISTS] = {NULL};
-  SDL_Surface* bkg = NULL;
-  SDL_Rect titleRects[8];
-  int stop = 0;
-  int loc = 0;
-  int old_loc = 1;
-
-  int themes = 0;
-  int i;
-  char fn[FNLEN];
-  char wordTypes[MAX_WORD_LISTS][FNLEN];
-  char fileNames[MAX_WORD_LISTS][FNLEN];
-
-  int old_use_english;
-  char old_theme_path[FNLEN];
-
-  FILE* fp;
-
-  DIR* themesDir = NULL;
-  struct dirent* themesFile = NULL;
-
- /* save previous settings in case we back out: */
-  old_use_english = settings.use_english;
-  strncpy(old_theme_path, settings.theme_data_path, FNLEN - 1);
-
-  if(settings.use_english)
-  {
-    sprintf(fn , "%s/words", settings.default_data_path);
-    fprintf(stderr , "%s/words", settings.default_data_path);
-  }
-  else
-  {
-    sprintf(fn , "%s/words", settings.theme_data_path);
-    fprintf(stderr , "%s/words", settings.theme_data_path);
-  }
-
-  themesDir = opendir(fn);
-
-  if (!themesDir)
-  {
-    fprintf(stderr, "Choosefile() - cannot open themes directory!");
-    return;
-  }
-
-  do
-  {
-    themesFile = readdir(themesDir);
-    if (!themesFile)
-      break;
-
-    /* we ignore any hidden file and CVS */
-
-    if (themesFile->d_name[0] == '.') 
-      continue;
-
-    if (strcmp("CVS", themesFile->d_name)==0)
-    continue;
-
-    if(settings.use_english)
-    {
-      sprintf(fn, "%s/words/%s",settings.default_data_path, themesFile->d_name);
-    }
-    else
-    {
-      sprintf(fn, "%s/words/%s", settings.theme_data_path, themesFile->d_name);
-    }
-
-    /* CheckFile() returns 2 if dir, 1 if file, 0 if neither: */
-    if (CheckFile(fn) == 1)
-    {
-      /* We know it opens safely because CheckFile() returned 1 */
-      fp = fopen(fn,"r");
-      /* HACK: we should get the names from file :) */
-      if (EOF ==fscanf(fp, "%[^\n]\n", wordTypes[themes]))
-        continue;
-      /* Make sure theme name is capitalized: */
-      wordTypes[themes][0] = toupper(wordTypes[themes][0]);
-      fclose(fp);
-      strncpy( fileNames[themes++], themesFile->d_name, FNLEN-1 );
-    }
-  } while (1);
-
-  closedir(themesDir);
-
-  settings.use_english = 1;
-        // HACK: is font empty now???
-
-  for (i = 0; i<themes; i++)
-  {
-    titles[i] = BlackOutline(wordTypes[i], DEFAULT_MENU_FONT_SIZE, &white);
-    select[i] = BlackOutline(wordTypes[i], DEFAULT_MENU_FONT_SIZE, &yellow);
-  }
-
-
-  settings.use_english = old_use_english;
-
-  bkg = LoadImage("main_bkg.png", IMG_REGULAR);
-
-
-  /* set initial rect sizes */
-  titleRects[0].y = 150;
-  titleRects[0].w = titleRects[0].h = titleRects[0].x = 0;
-  for (i = 1; i<8; i++)
-  {
-    titleRects[i].y = titleRects[i-1].y + 50;
-    titleRects[i].w = titleRects[i].h = titleRects[i].x = 0;
-  }
-
-
-  while (!stop)
-  {
-    while (SDL_PollEvent(&event))
-      switch (event.type)
-      {
-        case SDL_QUIT:
-          exit(0);
-          break;
-
-        case SDL_MOUSEMOTION:
-          for (i = 0; (i < 8) && (loc - (loc % 8) + i < themes); i++)
-            if (inRect(titleRects[i], event.motion.x, event.motion.y))
-            {
-              loc = loc - (loc % 8) + i;
-              break; /* out of for loop */
-            }
-          break;     /* out of switch-case */
-
-        case SDL_MOUSEBUTTONDOWN: 
-          for (i = 0; (i < 8) && (loc - (loc % 8) + i <themes); i++) 
-            if (inRect(titleRects[i], event.button.x, event.button.y))
-            {
-              loc = loc - (loc % 8) + i;
-              ChooseWord(fileNames[loc]);
-              break;
-            }
-          break;
-
-        case SDL_KEYDOWN:
-          if (event.key.keysym.sym == SDLK_ESCAPE)
-          {
-            settings.use_english = old_use_english;
-            strncpy(settings.theme_data_path, old_theme_path, FNLEN - 1);
-            stop = 1; 
-            break; 
-          }
-
-          if (event.key.keysym.sym == SDLK_RETURN) 
-          {
-            ChooseWord(fileNames[loc]);
-            loc = 0;
-            break;
-          }
-
-          if ((event.key.keysym.sym == SDLK_LEFT)
-           || (event.key.keysym.sym == SDLK_PAGEUP))
-          {
-            if (loc - (loc % 8) - 8 >= 0)
-              loc = loc - (loc % 8) - 8;
-          }
-
-          if ((event.key.keysym.sym == SDLK_RIGHT)
-           || (event.key.keysym.sym == SDLK_PAGEDOWN))
-          {
-            if (loc - (loc % 8) + 8 < themes)
-              loc = (loc - (loc % 8) + 8);
-          }
-
-          if (event.key.keysym.sym == SDLK_UP)
-          {
-            if (loc > 0)
-              loc--;
-          }
-
-          if (event.key.keysym.sym == SDLK_DOWN)
-          {
-            if (loc + 1 < themes)
-              loc++;
-          }
-      }
-
-    if (old_loc != loc)
-    {
-      int start;
-
-      SDL_BlitSurface( bkg, NULL, screen, NULL );
-
-      //if (loc) SetupPaths(fileNames[loc]); else SetupPaths(NULL);
-
-      start = loc - (loc % 8);
-
-      for (i = start; i < MIN (start + 8,themes); i++)
-      {
-        titleRects[i % 8].x = 320 - (titles[i]->w/2);
-        if (i == loc)
-          SDL_BlitSurface(select[loc], NULL, screen, &titleRects[i % 8]);
-        else
-          SDL_BlitSurface(titles[i], NULL, screen, &titleRects[i % 8]);
-      }
-
-      SDL_UpdateRect(screen, 0, 0, 0 ,0);
-    }
-    SDL_Delay(40);
-    old_loc = loc;
-  }
-
-  /* --- clear graphics before quitting --- */ 
-
-  for (i = 0; i<themes; i++)
-
-  {
-    SDL_FreeSurface(titles[i]);
-    SDL_FreeSurface(select[i]);
-  }
-
-  
-  SDL_FreeSurface(bkg);
- 
-  bkg = NULL;  /* the other pointers are going out of scope so we don't */
-               /* have to worry about setting them to NULL              */
-}
-
-
-static void ChooseWord(char *words_file)
-{
-  SDL_Surface* titles[MAX_WORD_LISTS] = {NULL};
-  SDL_Surface* select[MAX_WORD_LISTS] = {NULL};
-  SDL_Surface* left = NULL, *right = NULL;
-  SDL_Rect leftRect, rightRect;
-  SDL_Surface* bkg = NULL;
-  SDL_Rect titleRects[8];
-  int stop = 0;
-  int loc = 0;
-  int old_loc = 1;
-  int result = 0;
-
-  FILE* fp = NULL;
-
-  int start,themes,themest = 0;
-  int i,len;
-  char fn[FNLEN];
-  char str[FNLEN];
-  char editWordW[MAX_WORD_LISTS][FNLEN];
-  char editWordY[MAX_WORD_LISTS][FNLEN];
-
-  wchar_t temp[FNLEN];
-
-  int old_use_english;
-  char old_theme_path[FNLEN];
-
-  /* save previous settings in case we back out: */
-  old_use_english = settings.use_english;
-  strncpy(old_theme_path, settings.theme_data_path, FNLEN - 1);
-
-  if(settings.use_english)
-  {
-    sprintf(fn , "%s/words/%s", settings.default_data_path,words_file);
-  }
-  else
-  {
-    sprintf(fn , "%s/words/%s", settings.theme_data_path,words_file);
-  }
-
-  fp = fopen(fn,"r");
-  result = fscanf(fp, "%[^\n]\n", str);
-
-  while(!feof(fp))
-  {
-    /* HACK: we should get the strings from file :) */
-    if (EOF ==fscanf(fp, "%[^\n]\n", editWordW[themes]))
-      continue;
-    themest=themes;
-    strcpy(editWordY[themes++],editWordW[themest]);
-  }
-
-  fclose(fp); 
-
-  settings.use_english = 1;
-
-  for (i = 0; i<themes; i++)
-  {
-    titles[i] = BlackOutline(editWordW[i], DEFAULT_MENU_FONT_SIZE, &white);
-    strcat(editWordY[i],"|");
-    select[i] = BlackOutline(editWordY[i], DEFAULT_MENU_FONT_SIZE, &yellow);
-  }
-
-  settings.use_english = old_use_english;
-  strncpy(settings.theme_data_path, old_theme_path, FNLEN - 1);
-
-  bkg = LoadImage("main_bkg.png", IMG_REGULAR);
-
-  left = LoadImage("left.png", IMG_ALPHA);
-  leftRect.w = left->w; leftRect.h = left->h;
-  leftRect.x = 320 - 100 - (leftRect.w/2); leftRect.y = 430;
-
-  right = LoadImage("right.png", IMG_ALPHA);
-  rightRect.w = right->w; rightRect.h = right->h;
-  rightRect.x = 320 + 100 - (rightRect.w/2); rightRect.y = 430;
-
-	/* set initial rect sizes */ 
-  titleRects[0].y = 30;
-  titleRects[0].w = titleRects[0].h = titleRects[0].x = 0;
-  for (i = 1; i<8; i++)
-  {
-    titleRects[i].y = titleRects[i-1].y + 50;
-    titleRects[i].w = titleRects[i].h = titleRects[i].x = 0;
-  }
-
-
-  while (!stop) 
-  {
-   while (SDL_PollEvent(&event)) 
-    switch (event.type)
-    {
-      case SDL_QUIT:
-        exit(0);
-        break;
-
-      case SDL_MOUSEBUTTONDOWN: 
-        if (inRect(leftRect, event.button.x, event.button.y)) 
-          if (loc - (loc % 8) - 8 >= 0) 
-          {
-            loc = loc - (loc % 8) - 8;
-            break;
-          }
-
-        if (inRect(rightRect, event.button.x, event.button.y)) 
-          if (loc - (loc % 8) + 8 < themes)
-          {
-            loc = loc - (loc % 8) + 8;
-            break;
-          }
-
-        for (i = 0; (i < 8) && (loc - (loc % 8) + i < themes); i++)
-          if (inRect(titleRects[i], event.motion.x, event.motion.y))
-          {
-            loc = loc - (loc % 8) + i;
-             break;
-          }
-        break;
-
-      case SDL_KEYDOWN:
-        i = 1;
-
-        if (event.key.keysym.sym == SDLK_BACKSPACE)
-        {
-          len = ConvertFromUTF8(temp, editWordW[loc], FNLEN);
-          temp[len - 1] = temp[len];
-          len = ConvertToUTF8(temp, editWordW[loc], FNLEN);
-          titles[loc] = BlackOutline(editWordW[loc], DEFAULT_MENU_FONT_SIZE, &white );
-          len = ConvertFromUTF8(temp, editWordY[loc], FNLEN);
-          temp[len - 2] = temp[len - 1];
-          temp[len - 1] = temp[len];
-          len = ConvertToUTF8(temp, editWordY[loc], FNLEN);
-          select[loc] = BlackOutline(editWordY[loc], DEFAULT_MENU_FONT_SIZE, &yellow);
-          break;
-        }
-
-        if (event.key.keysym.sym == SDLK_ESCAPE) 
-        {
-          settings.use_english = old_use_english;
-          strncpy(settings.theme_data_path, old_theme_path, FNLEN - 1);
-          stop = 1;
-          break; 
-        }
-
-        if ((event.key.keysym.sym == SDLK_LEFT)
-         || (event.key.keysym.sym == SDLK_PAGEUP))
-        {
-          if (loc - (loc % 8) - 8 >= 0)
-            loc = loc - (loc % 8) - 8;
-          break;
-        }
-
-        if ((event.key.keysym.sym == SDLK_RIGHT)
-         || (event.key.keysym.sym == SDLK_PAGEDOWN))
-        {
-          if (loc - (loc % 8) + 8 < themes)
-            loc = (loc - (loc % 8) + 8);
-          break;
-        }
-
-        if (event.key.keysym.sym == SDLK_UP)
-        {
-          if (loc > 0)
-            loc--;
-          break;
-        }
-
-        if (event.key.keysym.sym == SDLK_DOWN)
-        {
-          if (loc + 1 < themes)
-            loc++;
-          break;
-        }
-
-        switch (event.key.keysym.sym)
-        {
-          case SDLK_CAPSLOCK:
-          case SDLK_RALT:
-          case SDLK_LALT:
-          case SDLK_RSHIFT:
-          case SDLK_LSHIFT:
-          case SDLK_RCTRL:
-          case SDLK_LCTRL:
-            i=0;
-            break;
-          default:  // ignore any other keys
-            {}
-        }
-
-        if(i)
-        {
-          len = ConvertFromUTF8(temp, editWordW[loc], FNLEN);
-          temp[len] = event.key.keysym.unicode;
-          temp[len + 1] = 0;
-          ConvertToUTF8(temp,editWordW[loc], FNLEN);
-          titles[loc] = BlackOutline(editWordW[loc], DEFAULT_MENU_FONT_SIZE, &white );
-          len = ConvertFromUTF8(temp,editWordY[loc], FNLEN);
-          temp[len + 1]=0;
-          temp[len] = temp[len-1];
-          temp[len - 1] = event.key.keysym.unicode;
-          ConvertToUTF8(temp,editWordY[loc], FNLEN);
-          select[loc] = BlackOutline(editWordY[loc], DEFAULT_MENU_FONT_SIZE, &yellow);
-          i = 0;
-          break;
-        }
-    }
-
-    if(!stop)
-    {
-      SDL_BlitSurface( bkg, NULL, screen, NULL );
-
-//	if (loc) SetupPaths(fileNames[loc]); else SetupPaths(NULL);	
-
-      start = loc - (loc % 8);
-      for (i = start; i < MIN(start + 8,themes); i++)
-      {
-        titleRects[i % 8].x = 320 - (titles[i]->w/2);
-        if (i == loc)
-          SDL_BlitSurface(select[loc], NULL, screen, &titleRects[i%8]);
-        else
-         SDL_BlitSurface(titles[i], NULL, screen, &titleRects[i%8]);
-      }
-
-      /* --- draw buttons --- */
-
-      if (start > 0) 
-        SDL_BlitSurface( left, NULL, screen, &leftRect );
-
-      if (start + 8 < themes) 
-        SDL_BlitSurface(right, NULL, screen, &rightRect);
-
-      SDL_UpdateRect(screen, 0, 0, 0 ,0);
-    }
-    //SDL_Delay(40);
-    old_loc = loc;
-  }
-
-  /* Write changes to file, if possible: */  
-  fprintf(stderr, "In ChooseWord(), about to write changes\n");
-  fp = fopen(fn,"w");
-
-  if (fp)
-  { 
-    fseek(fp, 0, SEEK_SET);
-    fprintf(fp, "%s\n", str);
-    i = 0;
-
-    while(i < themes)
-    {
-      fprintf(fp, "%s\n", editWordW[i++]);
-      i++;
-    }
-  
-    fclose(fp); 
-    fp = NULL;
-    fprintf(stderr, "In ChooseWord(), changes written successfully\n");
-  }
-  else
-  {
-    fprintf(stderr, "In ChooseWord(), unable to open %s for writing\n", fn);
-  }
-
-  
-  /* --- clear graphics before quitting --- */ 
-  /* FIXME none of this is safe if any of the images is NULL */
-  for (i = 0; i < themes; i++)
-  {
-    SDL_FreeSurface(titles[i]);
-    SDL_FreeSurface(select[i]);
-  }
-
-  SDL_FreeSurface(bkg);
-  SDL_FreeSurface(left);
-  SDL_FreeSurface(right);
-  bkg = NULL;  /* the other pointers are going out of scope so we don't */
-               /* have to worry about setting them to NULL              */
-}
